@@ -1,7 +1,7 @@
 /*
  * AIDE (Advanced Intrusion Detection Environment)
  *
- * Copyright (C) 1999-2002, 2006, 2011, 2015-2016, 2020 Rami Lehti,
+ * Copyright (C) 1999-2002, 2006, 2011, 2015-2016, 2020-2023 Rami Lehti,
  *               Pablo Virolainen, Richard van den Berg, Hannes von Haugwitz
  *
  * This program is free software; you can redistribute it and/or
@@ -21,19 +21,22 @@
 
 #ifndef _COMMANDCONF_H_INCLUDED
 #define _COMMANDCONF_H_INCLUDED
-#include "rx_rule.h"
-#include "list.h"
-#include <stdio.h>
+#include "config.h"
 #include <stdbool.h>
-#include "db_config.h"
+#include <stdio.h>
+#include "attributes.h"
+#include "config.h"
 #include "db.h"
+#include "db_config.h"
+#include "rx_rule.h"
+#include "seltree_struct.h"
 
 int parse_config(char *, char *, char *);
 
 int conf_input_wrapper(char* buf, int max_size, FILE* in);
 int db_input_wrapper(char*, int, database*);
 
-bool add_rx_rule_to_tree(char*, RESTRICTION_TYPE, DB_ATTR_TYPE, int, seltree*, int, char*, char*);
+bool add_rx_rule_to_tree(char*, char*, RESTRICTION_TYPE, DB_ATTR_TYPE, int, seltree*, int, char*, char*);
 
 void do_define(char*,char*, int, char*, char*);
 
@@ -51,7 +54,13 @@ void do_replevdef(char*);
 
 bool do_repurldef(char*, int, char*, char*);
 
-void do_rootprefix(char*, int, char*, char*);
+bool do_rootprefix(char*, int, char*, char*);
 
+#ifdef WITH_PTHREAD
+long do_num_workers(const char *);
+#endif
+
+#ifdef WITH_E2FSATTRS
 void do_report_ignore_e2fsattrs(char*, int, char*, char*);
 #endif
+#endif
