@@ -1,7 +1,7 @@
 /*
  * AIDE (Advanced Intrusion Detection Environment)
  *
- * Copyright (C) 1999-2002, 2011, 2021 Rami Lehti, Pablo Virolainen,
+ * Copyright (C) 1999-2002, 2011, 2021-2023 Rami Lehti, Pablo Virolainen,
  *               Hannes von Haugwitz
  *
  * This program is free software; you can redistribute it and/or
@@ -22,10 +22,14 @@
 #ifndef _DB_DISK_H_INCLUDED
 #define _DB_DISK_H_INCLUDED
 
-#include "db.h"
+#include "config.h"
 #include <stdbool.h>
+#include "db_config.h"
 
-db_line* db_readline_disk(bool);
-int db_disk_init(void);
+void db_scan_disk(bool);
 
+#ifdef WITH_PTHREAD
+int db_disk_start_threads();
+int db_disk_finish_threads();
+#endif
 #endif
