@@ -1,7 +1,7 @@
 /*
  * AIDE (Advanced Intrusion Detection Environment)
  *
- * Copyright (C) 1999-2002, 2006, 2020 Rami Lehti, Pablo Virolainen,
+ * Copyright (C) 1999-2002, 2006, 2020, 2022 Rami Lehti, Pablo Virolainen,
  *               Richard van den Berg, Hannes von Haugwitz
  *
  * This program is free software; you can redistribute it and/or
@@ -22,7 +22,12 @@
 #ifndef _DB_FILE_H_INCLUDED
 #define _DB_FILE_H_INCLUDED
 
+#include "config.h"
+#ifdef WITH_ZLIB
+#include <zlib.h>
+#endif
 #include "db_config.h"
+#include "url.h"
 
 char** db_readline_file(database*);
 int db_writespec_file(db_config*);
