@@ -1,7 +1,7 @@
 /*
  * AIDE (Advanced Intrusion Detection Environment)
  *
- * Copyright (C) 2000-2002, 2005-2006, 2020 Rami Lehti, Pablo Virolainen,
+ * Copyright (C) 2000-2002, 2005-2006, 2020, 2022 Rami Lehti, Pablo Virolainen,
  *               Richard van den Berg, Hannes von Haugwitz
  *
  * This program is free software; you can redistribute it and/or
@@ -22,16 +22,17 @@
 #ifndef _MD_H_INCLUDED
 #define _MD_H_INCLUDED
 
-#include "db_config.h"
-#include "hashsum.h"
-
+#include "config.h"
 #ifdef WITH_MHASH
 #include <mhash.h>
 #endif
-
 #ifdef WITH_GCRYPT
 #include <gcrypt.h>
 #endif
+#include <sys/types.h>
+#include "attributes.h"
+#include "hashsum.h"
+struct db_line;
 
 /*
   This struct hold's internal data needed for md-calls.
@@ -39,8 +40,6 @@
  */
 
 typedef struct md_container {
-  char hashsums[num_hashes][64];
-
   /* 
      Attr which are to be calculated.
   */
@@ -64,9 +63,14 @@ typedef struct md_container {
 
 } md_container;
 
+typedef struct md_hashsums {
+  char hashsums[num_hashes][HASHSUM_MAX_LENGTH];
+  DB_ATTR_TYPE attrs;
+} md_hashsums;
+
 int init_md(struct md_container*, const char*);
 int update_md(struct md_container*,void*,ssize_t);
-int close_md(struct md_container*);
-void md2line(struct md_container*,struct db_line*);
+int close_md(struct md_container*, md_hashsums *, const char*);
+void hashsums2line(md_hashsums*, struct db_line*);
 
 #endif /*_MD_H_INCLUDED*/
