@@ -1,7 +1,7 @@
 /*
  * AIDE (Advanced Intrusion Detection Environment)
  *
- * Copyright (C) 1999-2002, 2005-2006, 2010, 2019-2021 Rami Lehti,
+ * Copyright (C) 1999-2002, 2005-2006, 2010, 2019-2023 Rami Lehti,
  *               Pablo Virolainen, Richard van den Berg, Hannes von Haugwitz
  *
  * This program is free software; you can redistribute it and/or
@@ -19,13 +19,26 @@
  * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
-#include "aide.h"
+#include "config.h"
 #include <stdlib.h>
+#include <string.h>
+#include <sys/types.h>
+#include "attributes.h"
+#include "db_line.h"
+#include "db_config.h"
+#include "hashsum.h"
+#include "log.h"
 #include "md.h"
 #include "util.h"
-#include "log.h"
 #include "errorcodes.h"
-#include <string.h>
+
+#ifdef WITH_MHASH
+#include <mhash.h>
+#endif
+
+#ifdef WITH_GCRYPT
+#include <gcrypt.h>
+#endif
 
 /*
   Initialise md_container according its todo_attr field
@@ -60,12 +73,6 @@ int init_md(struct md_container* md, const char *filename) {
    }
 #endif 
 #ifdef WITH_GCRYPT
-  	if(!gcry_check_version(GCRYPT_VERSION)) {
-		log_msg(LOG_LEVEL_ERROR,"libgcrypt version mismatch");
-		exit(VERSION_MISMATCH_ERROR);
-	}
-	gcry_control(GCRYCTL_DISABLE_SECMEM, 0);
-	gcry_control(GCRYCTL_INITIALIZATION_FINISHED, 0);
 	if(gcry_md_open(&md->mdh,0,0)!=GPG_ERR_NO_ERROR){
 		log_msg(LOG_LEVEL_ERROR,"gcrypt_md_open failed");
 		exit(IO_ERROR);
@@ -84,7 +91,7 @@ int init_md(struct md_container* md, const char *filename) {
   }
 #endif
   char *str;
-  log_msg(LOG_LEVEL_DEBUG, " initialised md_container (%s) for '%s'", str = diff_attributes(0, md->calc_attr), filename);
+  log_msg(LOG_LEVEL_DEBUG, "%s> initialized md_container: %s (%p)", filename, str = diff_attributes(0, md->calc_attr), md);
   free(str);
   return RETOK;
 }
@@ -121,13 +128,13 @@ int update_md(struct md_container* md,void* data,ssize_t size) {
   After this calling update_db is not a good idea.
 */
 
-int close_md(struct md_container* md) {
+int close_md(struct md_container* md, md_hashsums * hs, const char *filename) {
 #ifdef _PARAMETER_CHECK_
   if (md==NULL) {
     return RETFAIL;
   }
 #endif
-  log_msg(LOG_LEVEL_DEBUG," free md_container");
+  log_msg(LOG_LEVEL_DEBUG, "%s> free md_container (%p)", filename, md);
 #ifdef WITH_MHASH
   for (HASHSUM i = 0 ; i < num_hashes ; ++i) {
       if(md->mhash_mdh[i] != MHASH_FAILED){
@@ -138,9 +145,11 @@ int close_md(struct md_container* md) {
 #ifdef WITH_GCRYPT
   gcry_md_final(md->mdh); 
 
-  for (HASHSUM i = 0 ; i < num_hashes ; ++i) {
-      if (md->calc_attr&ATTR(hashsums[i].attribute)) {
-          memcpy(md->hashsums[i],gcry_md_read(md->mdh, algorithms[i]), hashsums[i].length);
+  if (hs) {
+      for (HASHSUM i = 0 ; i < num_hashes ; ++i) {
+          if (md->calc_attr&ATTR(hashsums[i].attribute)) {
+              memcpy(hs->hashsums[i],gcry_md_read(md->mdh, algorithms[i]), hashsums[i].length);
+          }
       }
   }
 
@@ -150,10 +159,13 @@ int close_md(struct md_container* md) {
 #ifdef WITH_MHASH
   for (HASHSUM i = 0 ; i < num_hashes ; ++i) {
       if(md->mhash_mdh[i]!=MHASH_FAILED){
-          mhash_deinit(md->mhash_mdh[i],md->hashsums[i]);
+          mhash_deinit(md->mhash_mdh[i],hs?hs->hashsums[i]:NULL);
       }
   }
 #endif
+  if (hs) {
+      hs->attrs = md->calc_attr;
+  }
   return RETOK;
 }
 
@@ -161,7 +173,7 @@ int close_md(struct md_container* md) {
   Writes md_container to db_line.
  */
 
-void md2line(struct md_container* md,struct db_line* line) {
+void hashsums2line(md_hashsums *hs, struct db_line* line) {
   
 #ifdef _PARAMETER_CHECK_
   if (md==NULL||line==NULL) {
@@ -172,11 +184,12 @@ void md2line(struct md_container* md,struct db_line* line) {
    for (int i = 0 ; i < num_hashes ; ++i) {
        DB_ATTR_TYPE attr = ATTR(hashsums[i].attribute);
        if (line->attr&attr) {
-           if (md->calc_attr&attr) {
+           if (hs->attrs&attr) {
                line->hashsums[i] = checked_malloc(hashsums[i].length);
-               memcpy(line->hashsums[i],md->hashsums[i],hashsums[i].length);
+               memcpy(line->hashsums[i],hs->hashsums[i],hashsums[i].length);
            } else {
                line->attr&=~attr;
+               line->hashsums[i] = NULL;
            }
        } else {
             line->hashsums[i] = NULL;
